import factory
import swapper
from baseapp_comments.tests.factories import get_content_type, get_obj_pk
from baseapp_core.tests.factories import UserFactory
from django.contrib.contenttypes.models import ContentType

Reaction = swapper.load_model("baseapp_reactions", "Reaction")


class AbstractReactionFactory(factory.django.DjangoModelFactory):
    user = factory.SubFactory(UserFactory)
    reaction_type = factory.Faker("random_element", elements=Reaction.ReactionTypes)

    target_object_id = factory.LazyAttribute(get_obj_pk("target"))
    target_content_type = factory.LazyAttribute(get_content_type("target"))

    class Meta:
        exclude = ["target"]
        abstract = True

    def __setattr__(self, name, value):
        super().__setattr__(name, value)

        if name in ["target"]:
            setattr(self, f"{name}_content_type", ContentType.objects.get_for_model(value))
            setattr(self, f"{name}_object_id", value.id)


class ReactionFactory(AbstractReactionFactory):
    class Meta:
        model = Reaction



# Generated by Django 5.0.1 on 2024-06-07 00:32

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("testapp", "0002_user_preferred_language"),
        migrations.swappable_dependency(settings.BASEAPP_PROFILES_PROFILE_MODEL),
    ]

    operations = [
        migrations.AddField(
            model_name="user",
            name="profile",
            field=models.OneToOneField(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="%(class)s",
                to=settings.BASEAPP_PROFILES_PROFILE_MODEL,
                verbose_name="profile",
            ),
        ),
    ]



import swapper
from django.contrib import admin

Reaction = swapper.load_model("baseapp_reactions", "Reaction")


@admin.register(Reaction)
class ReactionAdmin(admin.ModelAdmin):
    search_fields = ("reaction_type", "user__first_name", "user__last_name")
    raw_id_fields = ("user",)
    list_display = ("id", "target", "user", "reaction_type")



import swapper
from django.contrib.auth.backends import BaseBackend

Reaction = swapper.load_model("baseapp_reactions", "Reaction")


class ReactionsPermissionsBackend(BaseBackend):
    def has_perm(self, user_obj, perm, obj=None):
        if perm == "baseapp_reactions.add_reaction":
            return user_obj.is_authenticated and getattr(obj, "is_reactions_enabled", False)

        if perm == "baseapp_reactions.view_reaction":
            # Anyone can view a reaction
            return True

        if perm in ["baseapp_reactions.change_reaction", "baseapp_reactions.delete_reaction"]:
            if isinstance(obj, Reaction):
                if obj.target and not getattr(obj.target, "is_reactions_enabled", True):
                    return False

                if user_obj.is_authenticated:
                    # Owner can change or delete their own reactions
                    if obj.user_id == user_obj.id:
                        return True

                    # Anyone with permission can change and delete any reaction
                    return user_obj.has_perm(perm)

        if perm == "baseapp_reactions.add_reaction_with_profile" and obj:
            return user_obj.has_perm("baseapp_profiles.use_profile", obj)



from baseapp_core.tests.settings import *  # noqa
from django.utils.translation import gettext_lazy as _

# Application definition
INSTALLED_APPS += [
    "graphene_django",
    "baseapp_notifications",
    "baseapp_reactions",
    "baseapp_comments",
    "baseapp_reports",
    "baseapp_auth",
    "baseapp_profiles",
    "baseapp_blocks",
    "baseapp_follows",
    "baseapp_pages",
    "testproject.testapp",
]

ROOT_URLCONF = "testproject.urls"

# Auth
AUTH_USER_MODEL = "testapp.User"

LANGUAGE_CODE = "en"
LANGUAGES = [("en", _("English")), ("es", _("Spanish")), ("pt", _("Portuguese"))]

AUTHENTICATION_BACKENDS = [
    "django.contrib.auth.backends.ModelBackend",
    "baseapp_auth.permissions.UsersPermissionsBackend",
    "baseapp_profiles.permissions.ProfilesPermissionsBackend",
    "baseapp_comments.permissions.CommentsPermissionsBackend",
    "baseapp_reactions.permissions.ReactionsPermissionsBackend",
]

BASEAPP_COMMENTS_ENABLE_NOTIFICATIONS = False

NOTIFICATIONS_NOTIFICATION_MODEL = "baseapp_notifications.Notification"



import pytest
from baseapp_comments.tests.factories import CommentFactory

from .factories import ReactionFactory

pytestmark = pytest.mark.django_db

VIEW_QUERY = """
    query GetObject($id: ID!) {
        node(id: $id) {
            ... on ReactionsInterface {
                reactionsCount {
                    total
                }
                reactions {
                    edges {
                        node {
                            id
                            pk
                        }
                    }
                }
            }
        }
    }
"""


def test_user_can_see_reactions(django_user_client, graphql_user_client):
    comment = CommentFactory(user=django_user_client.user)
    ReactionFactory(target=comment)
    response = graphql_user_client(VIEW_QUERY, variables={"id": comment.relay_id})
    content = response.json()
    assert content["data"]["node"]["reactionsCount"]["total"] == 1
    assert len(content["data"]["node"]["reactions"]["edges"]) == 1


# def test_user_can_see_reactions(django_user_instructor, graphql_user_instructor):
#     user = django_user_instructor.user
#     classroom = django_user_instructor.member.classroom
#     comment = ClassroomCommentFactory(target=classroom, user=user)
#     CommentReactionFactory(target=comment)
#     response = graphql_user_instructor(
#         "query { allClassrooms { totalCount edges { node { id commentsCount comments { edges { node { id content reactionsCount { total }}}} } } } }"
#     )
#     content = response.json()
#     assert content["data"]["allClassrooms"]["totalCount"] == 1
#     assert len(content["data"]["allClassrooms"]["edges"]) == 1
#     assert (
#         content["data"]["allClassrooms"]["edges"][0]["node"]["comments"]["edges"][0]["node"][
#             "reactionsCount"
#         ]["total"]
#         == 1
#     )



import django.db.models.deletion
import swapper
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("baseapp_reactions", "0002_alter_reaction_id"),
        swapper.dependency("baseapp_profiles", "Profile"),
    ]

    operations = [
        migrations.AddField(
            model_name="reaction",
            name="profile",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="reactions",
                to=swapper.get_model_name("baseapp_profiles", "Profile"),
                verbose_name="profile",
            ),
        ),
    ]



# Generated by Django 4.2.11 on 2024-04-16 01:15

import baseapp_core.models
import django.utils.timezone
import phonenumber_field.modelfields
from django.contrib.postgres.operations import CITextExtension
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ("auth", "0012_alter_user_first_name_max_length"),
    ]

    operations = [
        CITextExtension(),
        migrations.CreateModel(
            name="User",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                ("password", models.CharField(max_length=128, verbose_name="password")),
                (
                    "last_login",
                    models.DateTimeField(blank=True, null=True, verbose_name="last login"),
                ),
                (
                    "is_superuser",
                    models.BooleanField(
                        default=False,
                        help_text="Designates that this user has all permissions without explicitly assigning them.",
                        verbose_name="superuser status",
                    ),
                ),
                (
                    "email",
                    baseapp_core.models.CaseInsensitiveEmailField(
                        db_index=True, max_length=254, unique=True
                    ),
                ),
                ("is_email_verified", models.BooleanField(default=False)),
                (
                    "date_joined",
                    models.DateTimeField(
                        default=django.utils.timezone.now, verbose_name="date joined"
                    ),
                ),
                (
                    "password_changed_date",
                    models.DateTimeField(
                        default=django.utils.timezone.now,
                        verbose_name="last date password was changed",
                    ),
                ),
                (
                    "new_email",
                    baseapp_core.models.CaseInsensitiveEmailField(blank=True, max_length=254),
                ),
                (
                    "is_new_email_confirmed",
                    models.BooleanField(
                        default=False, help_text="Has the user confirmed they want an email change?"
                    ),
                ),
                ("first_name", models.CharField(blank=True, max_length=100)),
                ("last_name", models.CharField(blank=True, max_length=100)),
                (
                    "phone_number",
                    phonenumber_field.modelfields.PhoneNumberField(
                        blank=True, max_length=128, null=True, region=None, unique=True
                    ),
                ),
                (
                    "is_active",
                    models.BooleanField(
                        default=True,
                        help_text="Designates whether this user should be treated as active. Unselect this instead of deleting accounts.",
                        verbose_name="active",
                    ),
                ),
                (
                    "is_staff",
                    models.BooleanField(
                        default=False,
                        help_text="Designates whether the user can log into this admin site.",
                        verbose_name="staff status",
                    ),
                ),
                (
                    "groups",
                    models.ManyToManyField(
                        blank=True,
                        help_text="The groups this user belongs to. A user will get all permissions granted to each of their groups.",
                        related_name="user_set",
                        related_query_name="user",
                        to="auth.group",
                        verbose_name="groups",
                    ),
                ),
                (
                    "user_permissions",
                    models.ManyToManyField(
                        blank=True,
                        help_text="Specific permissions for this user.",
                        related_name="user_set",
                        related_query_name="user",
                        to="auth.permission",
                        verbose_name="user permissions",
                    ),
                ),
            ],
            options={
                "verbose_name": "user",
                "verbose_name_plural": "users",
                "permissions": [
                    ("view_all_users", "can view all users"),
                    ("view_user_email", "can view user's email field"),
                    ("view_user_phone_number", "can view user's phone number field"),
                    ("view_user_is_superuser", "can view user's is_superuser field"),
                    ("view_user_is_staff", "can view user's is_staff field"),
                    ("view_user_is_email_verified", "can view user's is_email_verified field"),
                    (
                        "view_user_password_changed_date",
                        "can view user's password_changed_date field",
                    ),
                    ("view_user_new_email", "can view user's new_email field"),
                    (
                        "view_user_is_new_email_confirmed",
                        "can view user's is_new_email_confirmed field",
                    ),
                ],
                "abstract": False,
            },
        ),
    ]



from baseapp_auth.models import AbstractUser
from baseapp_core.graphql.models import RelayModel
from baseapp_profiles.models import ProfilableModel
from django.utils import timezone
from model_utils import FieldTracker


class User(AbstractUser, RelayModel, ProfilableModel):
    # FieldTracker doesn't work with abstract model classes
    tracker = FieldTracker(fields=["is_superuser", "password"])

    def save(self, *args, **kwargs):
        with self.tracker:
            if self.tracker.has_changed("password"):
                self.password_changed_date = timezone.now()
            super().save(*args, **kwargs)



import graphene
import swapper
from baseapp_core.graphql import RelayMutation, get_obj_from_relay_id, login_required
from django.contrib.contenttypes.models import ContentType
from django.utils.translation import gettext_lazy as _
from graphql.error import GraphQLError
from graphql_relay.connection.arrayconnection import offset_to_cursor

from .object_types import ReactionObjectType, ReactionsInterface, ReactionTypesEnum

Reaction = swapper.load_model("baseapp_reactions", "Reaction")


class ReactionToggle(RelayMutation):
    reaction = graphene.Field(ReactionObjectType._meta.connection.Edge, required=False)
    target = graphene.Field(ReactionsInterface)
    reaction_deleted_id = graphene.ID(required=False)

    class Input:
        target_object_id = graphene.ID(required=True)
        profile_object_id = graphene.ID(required=False)
        reaction_type = graphene.Field(ReactionTypesEnum, required=True)

    @classmethod
    @login_required
    def mutate_and_get_payload(cls, root, info, **input):
        target = get_obj_from_relay_id(info, input.get("target_object_id"))
        target_content_type = ContentType.objects.get_for_model(target)
        reaction_type = input["reaction_type"]
        profile = None

        if not info.context.user.has_perm("baseapp_reactions.add_reaction", target):
            raise GraphQLError(
                str(_("You don't have permission to perform this action")),
                extensions={"code": "permission_required"},
            )

        if input.get("profile_object_id"):
            profile = get_obj_from_relay_id(info, input.get("profile_object_id"))
            if not info.context.user.has_perm(
                "baseapp_reactions.add_reaction_with_profile", profile
            ):
                raise GraphQLError(
                    str(_("You don't have permission to perform this action")),
                    extensions={"code": "permission_required"},
                )

        reaction, created = Reaction.objects.get_or_create(
            profile=profile,
            target_object_id=target.pk,
            target_content_type=target_content_type,
            defaults={"reaction_type": reaction_type, "user": info.context.user},
        )
        if not created:
            if reaction.reaction_type == reaction_type:
                if not info.context.user.has_perm("baseapp_reactions.delete_reaction", reaction):
                    raise GraphQLError(
                        str(_("You don't have permission to perform this action")),
                        extensions={"code": "permission_required"},
                    )

                reaction_deleted_id = reaction.relay_id
                reaction.delete()
                target.refresh_from_db()
                return ReactionToggle(target=target, reaction_deleted_id=reaction_deleted_id)

            if not info.context.user.has_perm("baseapp_reactions.change_reaction", reaction):
                raise GraphQLError(
                    str(_("You don't have permission to perform this action")),
                    extensions={"code": "permission_required"},
                )

            reaction.reaction_type = reaction_type
            reaction.save()

        target.refresh_from_db()

        return ReactionToggle(
            reaction=ReactionObjectType._meta.connection.Edge(
                node=reaction, cursor=offset_to_cursor(0)
            ),
            target=target,
        )


class ReactionsMutations(object):
    reaction_toggle = ReactionToggle.Field()



import graphene
from baseapp_auth.graphql.queries import UsersQueries
from baseapp_core.graphql import DeleteNode
from baseapp_profiles.graphql import ProfilesQueries
from graphene import relay

from baseapp_reactions.graphql.mutations import ReactionsMutations
from baseapp_reactions.graphql.queries import ReactionsQueries


class Query(graphene.ObjectType, ReactionsQueries, UsersQueries, ProfilesQueries):
    node = relay.node.NodeField(relay.Node)


class Mutation(
    graphene.ObjectType,
    ReactionsMutations,
):
    delete_node = DeleteNode.Field()


schema = graphene.Schema(query=Query, mutation=Mutation)



from baseapp_core.graphql import GraphQLView
from django.contrib import admin
from django.urls import path

__all__ = [
    "urlpatterns",
]

urlpatterns = [
    path("admin/", admin.site.urls),
    path("graphql", GraphQLView.as_view(graphiql=True)),
]



import swapper
from baseapp_core.graphql import RelayModel
from django.conf import settings
from django.contrib.contenttypes.fields import GenericForeignKey, GenericRelation
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.utils.translation import gettext_lazy as _
from model_utils.models import TimeStampedModel


def default_reactions_count():
    ReactionModel = swapper.load_model("baseapp_reactions", "Reaction")

    d = {"total": 0}

    for reaction_type in ReactionModel.ReactionTypes:
        d[reaction_type.name] = 0

    return d


class AbstractBaseReaction(TimeStampedModel, RelayModel):
    class ReactionTypes(models.IntegerChoices):
        LIKE = 1, _("like")
        DISLIKE = -1, _("dislike")

        @property
        def description(self):
            return self.label

    user = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        related_name="reactions",
        on_delete=models.CASCADE,
    )

    profile = models.ForeignKey(
        swapper.get_model_name("baseapp_profiles", "Profile"),
        verbose_name=_("profile"),
        related_name="reactions",
        on_delete=models.CASCADE,
        null=True,
        blank=True,
    )

    reaction_type = models.IntegerField(choices=ReactionTypes.choices, default=ReactionTypes.LIKE)

    target_content_type = models.ForeignKey(
        ContentType,
        blank=True,
        null=True,
        on_delete=models.CASCADE,
        db_index=True,
    )
    target_object_id = models.PositiveIntegerField(blank=True, null=True, db_index=True)
    target = GenericForeignKey("target_content_type", "target_object_id")

    class Meta:
        abstract = True
        indexes = [
            models.Index(fields=["target_content_type", "target_object_id"]),
        ]

    def __str__(self):
        return "Reaction (%s) #%s by %s" % (
            self.reaction_type,
            self.id,
            self.user.first_name,
        )

    def save(self, *args, **kwargs):
        super().save(*args, **kwargs)

        update_reactions_count(self.target)

    def delete(self, *args, **kwargs):
        super().delete(*args, **kwargs)

        update_reactions_count(self.target)


class Reaction(AbstractBaseReaction):
    class Meta:
        unique_together = [["user", "target_content_type", "target_object_id"]]
        swappable = swapper.swappable_setting("baseapp_reactions", "Reaction")


def update_reactions_count(target):
    if not target:
        return
    counts = default_reactions_count()
    ReactionModel = swapper.load_model("baseapp_reactions", "Reaction")
    target_content_type = ContentType.objects.get_for_model(target)
    for reaction_type in ReactionModel.ReactionTypes:
        # TO DO: improve performance by removing the FOR and making 1 query to return counts for all types at once
        counts[reaction_type.name] = ReactionModel.objects.filter(
            target_content_type=target_content_type,
            target_object_id=target.pk,
            reaction_type=reaction_type,
        ).count()
        counts["total"] += counts[reaction_type.name]

    target.reactions_count = counts
    target.save(update_fields=["reactions_count"])


SwappedReaction = swapper.load_model(
    "baseapp_reactions", "Reaction", required=False, require_ready=False
)


class ReactableModel(models.Model):
    reactions_count = models.JSONField(default=default_reactions_count, editable=False)
    reactions = GenericRelation(
        SwappedReaction,
        content_type_field="target_content_type",
        object_id_field="target_object_id",
    )
    is_reactions_enabled = models.BooleanField(default=True)

    class Meta:
        abstract = True



import graphene
import graphene_django_optimizer as gql_optimizer
import swapper
from baseapp_core.graphql import DjangoObjectType, get_pk_from_relay_id
from django.conf import settings
from django.contrib.contenttypes.models import ContentType
from graphene import relay
from graphene_django.filter import DjangoFilterConnectionField

Reaction = swapper.load_model("baseapp_reactions", "Reaction")
Profile = swapper.load_model("baseapp_profiles", "Profile")

ReactionTypesEnum = graphene.Enum.from_enum(Reaction.ReactionTypes)


def create_object_type_from_enum(name, enum):
    fields = {}
    for reaction_type in enum:
        fields[reaction_type.name] = graphene.Int()
    fields["total"] = graphene.Int()
    return type(name, (graphene.ObjectType,), fields)


ReactionsCount = create_object_type_from_enum("ReactionsCount", Reaction.ReactionTypes)


class ReactionsInterface(relay.Node):
    reactions_count = graphene.Field(ReactionsCount)
    reactions = DjangoFilterConnectionField(lambda: ReactionObjectType)
    is_reactions_enabled = graphene.Boolean(required=True)
    my_reaction = graphene.Field(
        lambda: ReactionObjectType,
        required=False,
        profile_id=graphene.ID(required=False),
    )

    def resolve_reactions(self, info, **kwargs):
        if not getattr(self, "is_reactions_enabled", True):
            return Reaction.objects.none()

        CAN_ANONYMOUS_VIEW_REACTIONS = getattr(
            settings, "BASEAPP_REACTIONS_CAN_ANONYMOUS_VIEW_REACTIONSS", True
        )
        if not CAN_ANONYMOUS_VIEW_REACTIONS and not info.context.user.is_authenticated:
            return Reaction.objects.none()

        target_content_type = ContentType.objects.get_for_model(self)
        return Reaction.objects.filter(
            target_content_type=target_content_type,
            target_object_id=self.pk,
        ).order_by("-created")

    def resolve_my_reaction(self, info, profile_id=None, **kwargs):
        if info.context.user.is_authenticated:
            if profile_id:
                pk = get_pk_from_relay_id(profile_id)
                profile = Profile.objects.get_if_member(pk=pk, user=info.context.user)
            else:
                profile = info.context.user.current_profile
            if not profile:
                return None
            return Reaction.objects.filter(
                target_content_type=ContentType.objects.get_for_model(self),
                target_object_id=self.pk,
                profile_id=profile.pk,
            ).first()


class ReactionObjectType(gql_optimizer.OptimizedDjangoObjectType, DjangoObjectType):
    target = graphene.Field(relay.Node)
    reaction_type = graphene.Field(ReactionTypesEnum)

    class Meta:
        interfaces = (relay.Node,)
        model = Reaction
        fields = (
            "id",
            "user",
            "reaction_type",
            "created",
            "modified",
            "target",
        )
        filter_fields = {
            "id": ["exact"],
        }

    @classmethod
    def get_node(self, info, id):
        node = super().get_node(info, id)
        if not info.context.user.has_perm("baseapp_comments.view_comment", node):
            return None
        return node



from baseapp_auth.admin import AbstractUserAdmin
from django.contrib import admin
from django.contrib.auth import get_user_model

User = get_user_model()


@admin.register(User)
class UserAdmin(AbstractUserAdmin):
    pass



# Generated by Django 4.2.13 on 2024-06-28 00:26

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("testapp", "0001_initial"),
    ]

    operations = [
        migrations.AddField(
            model_name="user",
            name="preferred_language",
            field=models.CharField(
                choices=[("en", "English"), ("es", "Spanish"), ("pt", "Portuguese")],
                default="en",
                max_length=9,
            ),
        ),
    ]



# Generated by Django 3.2.19 on 2023-06-16 12:47

import django.db.models.deletion
import django.utils.timezone
import model_utils.fields
import swapper
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("contenttypes", "0002_remove_content_type_name"),
        swapper.dependency("baseapp_reactions", "Reaction"),
    ]

    operations = [
        migrations.CreateModel(
            name="Reaction",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "created",
                    model_utils.fields.AutoCreatedField(
                        default=django.utils.timezone.now,
                        editable=False,
                        verbose_name="created",
                    ),
                ),
                (
                    "modified",
                    model_utils.fields.AutoLastModifiedField(
                        default=django.utils.timezone.now,
                        editable=False,
                        verbose_name="modified",
                    ),
                ),
                (
                    "reaction_type",
                    models.IntegerField(choices=[(1, "like"), (-1, "dislike")], default=1),
                ),
                (
                    "target_object_id",
                    models.PositiveIntegerField(blank=True, db_index=True, null=True),
                ),
                (
                    "target_content_type",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to="contenttypes.contenttype",
                    ),
                ),
                (
                    "user",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="reactions",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={
                "swappable": swapper.swappable_setting("baseapp_reactions", "Reaction"),
                "unique_together": {("user", "target_content_type", "target_object_id")},
            },
        ),
    ]



from setuptools import find_packages, setup

with open("README.md", "r") as f:
    long_description = f.read()

setup(
    name="baseapp-reactions",
    packages=find_packages(),
    long_description=long_description,
    long_description_content_type="text/markdown",
)



# Generated by Django 3.2 on 2023-08-21 15:41

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("baseapp_reactions", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="reaction",
            name="id",
            field=models.AutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
    ]



from django.apps import AppConfig


class ReactionsConfig(AppConfig):
    default = True
    name = "baseapp_reactions"
    verbose_name = "BaseApp Reactions"
    default_auto_field = "django.db.models.AutoField"



import pytest
import swapper
from baseapp_comments.tests.factories import CommentFactory
from baseapp_profiles.tests.factories import ProfileFactory

from baseapp_reactions.tests.factories import ReactionFactory

Reaction = swapper.load_model("baseapp_reactions", "Reaction")
ReactionTypes = Reaction.ReactionTypes

pytestmark = pytest.mark.django_db

REACTION_TOGGLE_GRAPHQL = """
    mutation ReactionToggleMutation($input: ReactionToggleInput!) {
        reactionToggle(input: $input) {
            reaction {
                node {
                    id
                    reactionType
                }
            }
        }
    }
"""


def test_anon_cant_react(graphql_client):
    comment = CommentFactory()

    response = graphql_client(
        REACTION_TOGGLE_GRAPHQL,
        variables={"input": {"targetObjectId": comment.relay_id, "reactionType": "LIKE"}},
    )
    content = response.json()
    assert content["errors"][0]["message"] == "authentication required"
    assert Reaction.objects.count() == 0


def test_user_can_add_reaction(graphql_user_client):
    comment = CommentFactory()

    # create reaction with type LIKE
    graphql_user_client(
        REACTION_TOGGLE_GRAPHQL,
        variables={
            "input": {
                "targetObjectId": comment.relay_id,
                "reactionType": ReactionTypes.LIKE.name,
            }
        },
    )
    assert Reaction.objects.count() == 1


def test_user_can_change_reaction(django_user_client, graphql_user_client):
    comment = CommentFactory()
    reaction = ReactionFactory(
        target=comment, user=django_user_client.user, reaction_type=ReactionTypes.LIKE
    )

    # change reaction with type LIKE to DISLIKE
    graphql_user_client(
        REACTION_TOGGLE_GRAPHQL,
        variables={
            "input": {
                "targetObjectId": comment.relay_id,
                "reactionType": ReactionTypes.DISLIKE.name,
            }
        },
    )

    comment.refresh_from_db()
    reaction.refresh_from_db()
    assert reaction.reaction_type == ReactionTypes.DISLIKE
    assert comment.reactions_count["total"] == 1
    assert comment.reactions_count["LIKE"] == 0
    assert comment.reactions_count["DISLIKE"] == 1


def test_user_can_remove_reaction(django_user_client, graphql_user_client):
    comment = CommentFactory()
    ReactionFactory(
        target=comment, user=django_user_client.user, reaction_type=ReactionTypes.DISLIKE
    )
    # remove reaction
    graphql_user_client(
        REACTION_TOGGLE_GRAPHQL,
        variables={
            "input": {
                "targetObjectId": comment.relay_id,
                "reactionType": ReactionTypes.DISLIKE.name,
            }
        },
    )
    assert Reaction.objects.count() == 0
    comment.refresh_from_db()
    assert comment.reactions_count["total"] == 0
    assert comment.reactions_count["LIKE"] == 0
    assert comment.reactions_count["DISLIKE"] == 0


def test_user_can_react_with_profile(django_user_client, graphql_user_client):
    profile = ProfileFactory(owner=django_user_client.user)
    target = CommentFactory()

    graphql_user_client(
        REACTION_TOGGLE_GRAPHQL,
        variables={
            "input": {
                "targetObjectId": target.relay_id,
                "reactionType": ReactionTypes.DISLIKE.name,
                "profileObjectId": profile.relay_id,
            }
        },
    )
    assert Reaction.objects.count() == 1


def test_user_cant_react_with_profile(graphql_user_client):
    profile = ProfileFactory()
    target = CommentFactory()

    response = graphql_user_client(
        REACTION_TOGGLE_GRAPHQL,
        variables={
            "input": {
                "targetObjectId": target.relay_id,
                "reactionType": ReactionTypes.DISLIKE.name,
                "profileObjectId": profile.relay_id,
            }
        },
    )
    content = response.json()
    assert content["errors"][0]["extensions"]["code"] == "permission_required"
    assert Reaction.objects.all().count() == 0
